/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.lang.IllegalArgumentException;
import java.lang.NumberFormatException;

import java.lang.Double;
import java.lang.Integer;

import java.lang.Runnable;
import java.lang.Thread;

import noope.core.Vector3D;
import noope.core.Physics;

import noope.input.StringBlockReader;
import noope.input.ArrayBlockReader;
import noope.input.BlockReader;
import noope.input.BRLoadingException;

import noope.output.NullOutputStream;

import java.io.PrintStream;

/**
 *
 * This program produces an animated gif showing the simulation of spheres under gravity and
 * elastic collision.
 *
 * It takes command line arguments.
 *
 * It outputs an estimate of the total running time to stdout, followed by an eoln character.
 *
 *
 */
public class MakeAnimGif implements Runnable
{
    private double
	totalTime,
	dt;
    private String samplingPeriod;
    private ProjectionBrData proj;
    private String[] projNames;
    private EntityBrData[] ents;

    private String gifName;

    private Physics phys;

    private volatile boolean simulationStopped;
    
    /**
     * Command-line arguments:
     *
     * gifName: fileName String
     * totalSimulationTime: double
     * dt: double
     * normal: Vector3D
     * sampling period of OutputProjection: double
     * xmin, xmax, ymin, ymax: doubles
     * n: int -- number of Entities
     *
     * 
     * <br><br>
     * And n times: 
     * position: Vector3D
     * velocity: Vector3D
     * mass: double
     * radius: double
     * colour: name or Vector3D
     *
     */
    public static void main(String[] args)
    {
	MakeAnimGif thisObject = new MakeAnimGif();

	ArrayBlockReader br = null;
	try
	    {
		thisObject.getArgs(args);
	    }
	catch (IllegalArgumentException e)
	    {
		e.printStackTrace();
		System.exit(1);
	    }

	try
	    {
		br = thisObject.getPhysBr();
	    }
	catch (IllegalStateException e)
	    {
		e.printStackTrace();
		System.exit(1);
	    }

	try
	    {
		thisObject.phys = new Physics(br);
		thisObject.simulationStopped = false;
		(new Thread(thisObject)).start();
		try
		    {
			Thread.sleep(10000L); // sleep for 10 seconds
		    }
		catch (InterruptedException e)
		    { // we can't really do anything about it
		    }
		if (thisObject.simulationStopped)
		    {
			System.out.println("0.0");
			System.exit(0);
		    }
		else
		    System.out.println((1.0F - (float)(thisObject.phys.getTime() 
						       / thisObject.totalTime)) * 10.0F
				       + 4.0F);
	    }
	catch (BRLoadingException e)
	    {
		e.printStackTrace();
		System.exit(1);
	    }
    }

    /** Calls phys.run(totalTime, dt), followed by phys.dispose(). */
    public void run()
    {
	phys.run(totalTime, dt);
	phys.dispose();
	simulationStopped = true;
    }

    /** returns a BlockReader object representing a Physics object. */
    private ArrayBlockReader getPhysBr() throws IllegalStateException
    {
	String header = "physics", parameter = null;
	String[][] entries = null;
	ArrayBlockReader[] subblocks = {getLawsBr(getLawBrs()), getEntitiesBr(getEntityBrs())};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** returns an array of BlockReader objects representing a Gravity, a SphereCollision and various
	OutputProjection laws. */
    private ArrayBlockReader[] getLawBrs() throws IllegalStateException
    {
	int i = 0; // counter variable for FOR-loop
	// get G
	String sG = "6.6732e-11";
		
	ArrayBlockReader[] result = new ArrayBlockReader[3];

	result[0] = newGravBr(sG);
	result[1] = newSphereCollBr();

	try
	{
	    result[2] = proj.getAnimGifArrayBlockReader(gifName, samplingPeriod);
	}
	catch (IllegalArgumentException e)
	{
	    throw new IllegalStateException("Could not create ArrayBlockReader representing " +
					    "OutputProjection.");
	}

	return result;
    }

    /** returns an array of BlockReaders object representing the various entities. */
    private ArrayBlockReader[] getEntityBrs() throws IllegalStateException
    {
	int i = 0; // counter variable for FOR loop
	
	ArrayBlockReader[] result = new ArrayBlockReader[ents.length];
	
	try
	{
	    for (i = 0; i < ents.length; i++)
		{
		    result[i] = ents[i].getArrayBlockReader(projNames, 1);
		}
	}
	catch (IllegalArgumentException e)
	{
	    throw new IllegalStateException("Could not create ArrayBlockReader representing " +
					    "Entity.");
	}
	return result;
    }

    private ArrayBlockReader getEntitiesBr(ArrayBlockReader[] subblocks)
    {
	String header = "entities", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    // creates the laws {...} block containing all the laws as subblocks
    private ArrayBlockReader getLawsBr(ArrayBlockReader[] subblocks)
    {
	String header = "laws", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    private ArrayBlockReader newGravBr(String sG) // creates an ArrayBlockReader defining a Gravity law
    {
	String header = "law", parameter = "Gravity"; 
	String[][] entries = 
	{{"name", "Gravity"},
	 {"G", sG}
	};
	ArrayBlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** Create a BlockReader object to construct a SphereCollision law from */
    private ArrayBlockReader newSphereCollBr()
    {
	String header = "law", parameter = "SphereCollision"; 
	String[][] entries = 
	{{"name", "SphereCollision"}
	};
	ArrayBlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }


    private void getArgs(String[] args) throws IllegalArgumentException
    {
	String entPos;
	String entVel;
	String entMass;
	String entRadius;
	String entColor;

	int argnum = 0;

	// make sure first arguments provided
	if (args.length < 9)
	    {
		throw new IllegalArgumentException("Too few arguments provided.");
	    }
	try
	    {
		gifName = args[argnum];
		argnum++;
		
		totalTime = Double.parseDouble(args[argnum]);
		argnum++;
		
		dt = Double.parseDouble(args[argnum]);
		argnum++;
		
		proj = new ProjectionBrData();
		proj.setName("Projection 1");
		projNames = new String[]{proj.getName()};

		proj.setNormal(args[argnum]);
		argnum++;

		samplingPeriod = args[argnum];
		Double.valueOf(samplingPeriod);
		argnum++;

		proj.setXmin(args[argnum]);
		argnum++;

		proj.setXmax(args[argnum]);
		argnum++;

		proj.setYmin(args[argnum]);
		argnum++;

		proj.setYmax(args[argnum]);
		argnum++;
		
		if (!(Integer.parseInt(args[argnum]) > 0))
		    throw new NumberFormatException();
		ents = new EntityBrData[Integer.parseInt(args[argnum])];
		argnum++;

		// now make sure we've got a sufficient number of arguments
		if (args.length != (argnum + (5 * ents.length)))
		    {
			throw new IllegalArgumentException("Too few arguments provided.");
		    }

		// now get the Entity-s
		for (int i = 0; i < ents.length; i++)
		    {
			ents[i] = new EntityBrData();
			ents[i].setName("Entity " + (i + 1));
			
			entPos = args[argnum];
			ents[i].setPosition(entPos);
			argnum++;

			entVel = args[argnum];
			ents[i].setVelocity(entVel);
			argnum++;

			entMass = args[argnum];
			ents[i].setMass(entMass);
			argnum++;

			entRadius = args[argnum];
			ents[i].setRadius(entRadius);
			argnum++;

			entColor = args[argnum];
			ents[i].setColor(entColor);
			argnum++;
		    }
	    }
	catch (NumberFormatException e)
	    {
		e.printStackTrace();
		throw new IllegalArgumentException("Failed to parse argument " + (argnum + 1) 
						   +  ", " + args[argnum] + ".");
	    }
    }
}
