/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import javax.swing.*;
import noope.core.Vector3D;

/**
 * This is a subclass of InputVerifier that can be used to validate a
 * JTextField. The contents are valid if the text can be interpreted as a
 * Vector3D (using the Vector3D constructor taking a String).
 *
 * @author Miklos Reiter
 * @version 1.0
 */

public class Vector3DInputVerifier extends InputVerifier {

  /**
   * Whether the Vector3DInputVerifier accepts the empty string as a valid
   * input.
   */
  private boolean acceptEmpty;

  /**
   * Constructs a new Vector3DInputVerifier object.
   *
   * This verifier will accept the empty string as a valid entry.
   */
  public Vector3DInputVerifier() {
    acceptEmpty = true;
  }

  /**
   * Constructs a new Vector3DInputVerifier object.
   *
   * @arg acceptEmpty Whether the empty string should be accepted as
   * valid.
   */
  public Vector3DInputVerifier(boolean emptyEntryAccepted) {
    acceptEmpty = emptyEntryAccepted;
  }

  /**
   * Is the empty string accepted as a valid entry?
   */
  public boolean getAcceptEmpty()
  {
    return acceptEmpty;
  }

  /**
   * Sets whether the empty string should be accepted as valid.
   */
  public void setAcceptEmpty(boolean emptyEntryAccepted)
  {
    acceptEmpty = emptyEntryAccepted;
  }


  /**
   * Displays a message box complaining to the user and informing them
   * briefly of the format of a Vector3D, if the input is invalid, in
   * addition to calling the inherited method.
   */
  public boolean shouldYieldFocus(JComponent input)
  {
    boolean result = super.shouldYieldFocus(input);
    if (!result)
      JOptionPane.showMessageDialog(input, "Please enter a valid " +
        "Vector3D: '(x,y,z)' where x, y and z are doubles or the value"
        + " '0'.", "Error", JOptionPane.ERROR_MESSAGE);
    return result;
  }

  /**
   * Casts the argument as a JTextField, then parses it as a Vector3D.
   * Returns true if it succeeds, false otherwise. Calls validVector3D
   * to do this. True is also returned if s is the empty String and
   * getAcceptEmpty is true.
   */
  public boolean verify(JComponent input) {
    String s = ((JTextField)input).getText();
    if (getAcceptEmpty() && s.equals("")) return true;
    return validVector3D(s);
  }

  /**
   * Returns true if the argument represents a valid Vector3D. Uses
   * the constructor of the Vector3D class taking a String.
   */
  public static boolean validVector3D(String s)
  {
    try
    {
      Vector3D v = new Vector3D(s);
    }
    catch (java.text.ParseException e)
    {
      return false;
    }
    return true; // no ParseException occurred
  }
}
