/**
 * Copyright (C) 2001  James Kermode (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


package noope.core;

import java.util.Vector;
//import java.util.ListIterator;
import java.lang.reflect.Constructor;

import noope.input.BlockReader;
import noope.input.BlockReaderLocation;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;
import noope.input.BRClassLoaderUtils;
import noope.input.EntityLoader;

import noope.entities.Entity;

/** 
 * This class maintains a list of Entity objects.
 * 
 * It is a subclass of Vector, and as such it implements 
 * the List interface. The Entity objects are acted on by Law objects, 
 * but this requires no special support from the Entities class.
 *
 * @author James Kermode
 * @version 28-Apr-2001 Miklos Reiter: removed ListIterator for compatibility with Java 1.1
 */
public class Entities extends Vector 
{
   /**
    * Constructs the Entities object from the supplied BlockReader.
    *
    * This should be the entities{...} block from the main BlockReader - this
    * constructor will only be called by the Physics(BlockReader) constructor.
    * The physics object is not used, it is just passed along to each
    * of the Entity() constructors.
    *
    * @param br BlockReader used to read in the Entity objects
    * @param phys Physics object passed on to Entity() constructors
    * @see noope.entities.Entity#Entity(BlockReader, Physics)
    * @exception BRLoadingException if invalid data is read from the BlockReader
    */
    public Entities(BlockReader br, Physics phys) throws BRLoadingException
    {

	EntityLoader entityLoader = phys.getEntityLoader();

	if (!br.hasMoreBlocks())
	    throw new BRLoadingParseException("Can't construct Entities from empty BlockReader", 
	                                       br.getBlockLocation());

	BlockReaderLocation brL = null; // for BR*exceptions.
	
	// Load entities one at a time from the BlockReader
	BlockReader brEntity;
	String entityClassName; // Class file to construct, defaults to Entity

	while (br.hasMoreBlocks())
	{
	    brEntity = br.getNextBlock();
	    // location to use if error occurs while loading class
	    brL = br.getBlockLocation();
	    if (!brEntity.getHeader().equals("entity")) // Check it's an entity block
		throw new BRLoadingParseException("Non-entity block found in entities{...}", 
						  brL);

	    // Set className to the block parameter, or to "Entity" if there is none
	    entityClassName = (((brEntity.getParameter() == null) 
				|| (brEntity.getParameter().equals("")) ? 
				"Entity" : brEntity.getParameter()));
	    
	    this.add(entityLoader.createEntity(entityClassName, brL, brEntity, phys));
	    
	    brEntity.assertEmpty();
	}
	br.assertEmpty();
    }

    /** Disposes of resources by calling dispose() on each of the Entity objects stored in the
	List. Added by Miklos Reiter on 19 April 2001. */
    public void dispose()
    {
	Entity entity;
	// For each entity, call dispose() method
	
	for (int i = 0; i < size(); i++)
	    {
		((Entity)elementAt(i)).dispose();
	    }

	// remove all elements now
	this.removeAllElements();
    }
}
