/**
 * Copyright (C) 2001  Paul Evans (see the file README for details on how
 * to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.core;

import java.text.ParseException; // thrown by Vector3D(String)
import java.util.StringTokenizer; // used to parse the String in Vector3D(String)
import java.lang.Double; // parseDouble

/**
 * Manipulate a 3D cartesian vector.
 * Note that objects of this class are immutable; i.e. they cannot be changed. To create
 * mutable (i.e. changable) vectors, use the MutableVector3D class instead.
 *
 * @author Paul Evans
 * @version Modified by James Kermode, 22-Mar-2001, again by Paul Evans, 28-Mar-2001
 */
public class Vector3D
{
    // Changed to protected by Paul, 28/03 so MutableVector3D can see them

    /** x component */
    protected double x;
 
    /** y component */
    protected double y;
    
    /** z component */
    protected double z;

    /** Unit vector in x direction */
    public static final Vector3D ex = new Vector3D(1, 0, 0);
    /** Unit vector in y direction */
    public static final Vector3D ey = new Vector3D(0, 1, 0);
    /** Unit vector in z direction */
    public static final Vector3D ez = new Vector3D(0, 0, 1);
    
    /** Construct a Vector3D and initialise it to (0, 0, 0). */
    public Vector3D()
    {
	x = y = z = 0;
    }
    
    /** Construct a Vector3D and initialise it to (new_x, new_y, new_z).
     * @param new_x x component
     * @param new_y y component
     * @param new_z z component
     */
    public Vector3D(double new_x, double new_y, double new_z)
    {
	x = new_x;
	y = new_y;
	z = new_z;
    }
    
    /** Construct a Vector3D equal to <i>v</i>.
     * @param v the vector to copy from
     */
    public Vector3D(Vector3D v)
    {
	x = v.x;
	y = v.y;
	z = v.z;
    }
    
    /** Construct a new Vector3D by parsing a string.
     * Parses the String <i>s</i> for a Vector3D given in the form (x, y, z) where
     * <i>x</i>, <i>y</i> and <i>z</i> are doubles. All whitespace is stripped and "0" is recognised
     * to mean the zero vector (0, 0, 0).
     * Originially written by Miklos Reiter, relocated and minor modifications by James Kermode
     * @param s the string to parse
     * @throws ParseException if there was an error parsing the string
     */
    public Vector3D(String s) throws ParseException
    {
    	final int DIMENSION = 3; // constant for maintainability
	
	// check whether s is "0" first: this has to be promoted to (0,0,0).
	if (s.equals("0"))
	{
	   x = y = z = 0;
	   return;
	}

	// make a StringTokenizer that treats as delimiters all whitespace,
	// ()'s and ",". It should return the delimiters as tokens.
	StringTokenizer st = new StringTokenizer(s, "\t\n\r\f(),", true);
	double v[] = new double[DIMENSION]; // coordinates of vector
	boolean started = false, ended = false; // has a ( and a ) occurred
	// yet?
	boolean isDelimiter;
	int coordinate = 0; // which coordinate are we at?
	String token; // current token

	while (st.hasMoreTokens())
	{
	    token = st.nextToken();
	    isDelimiter = false; // we don't know yet, so assume default
	    // whitespace or ","? skip over it!
	    if (token.length() == 1)
	    {
		switch (token.charAt(0))
		{
		case '\t': // drop through
		case '\n': // drop through
		case '\r': // drop through
		case '\f': // drop through
		case ',': 
			isDelimiter = true;
			break; // ignore whitespace and ","
		case '(':
			if (started)
			throw new ParseException("Error occurred while parsing for Vector3D",0);
			else started = true;
			isDelimiter = true;
			break;
		case ')': 
			if (ended)
			throw new ParseException("Error occurred while parsing for Vector3D",0);
			else ended = true;
			isDelimiter = true;
			break;
		default: isDelimiter = false;
		} // end switch
	    } // end if
	    // otherwise we must have a double
	    if (started && (!ended) && (!isDelimiter)) 
	    {
		// try to interpret token as a double
		try
		{
		    if (coordinate < DIMENSION)
		    v[coordinate] =
			    Double.valueOf(token).doubleValue();
		    else throw new ParseException("Error occurred while parsing for Vector", 0);
		}
		catch (NumberFormatException n)
		{
		    throw new ParseException("Error occurred while parsing for Vector", 0);
		}
		coordinate++; // move on to next coordinate
	    }
	} // end while
	// now try to create the Vector3D
	if ((coordinate < DIMENSION) || (!ended))
	{
	    throw new ParseException("Error occurred while parsing for Vector", 0);
	}

	x = v[0];
	y = v[1];
	z = v[2];
    }

    /** Hash this Vector3D to an integer. */
    public int hashCode()
    {
	return (int)(x + 2.51*y + 624.43*z);  // random, arbitrary values
    }

    /** Compare a Vector3D and an Object.
     * @returns <i>false</i> unless o is a Vector3D, in which case it calls Vector3D.equals(Vector3D).
     * @see #equals(Vector3D)
     */
    public boolean equals(Object o)
    {
	if (!(o instanceof Vector3D)) return false;
	return equals((Vector3D)o);
    }

    /** Compare two Vector3D objects.
     * @return <i>true</i> if the x, y, and z components of the two vectors are equal,
     * <i>false</i> otherwise. */
    public boolean equals(Vector3D v)
    {
	return (x == v.x) && (y == v.y) && (z == v.z);
    }
    
    /** Return the x component of this Vector3D */
    public double getX()
    {
        return x;
    }
    
    /** Return the y component of this Vector3D */
    public double getY()
    {
    	return y;
    }
    
    /** Return the z component of this Vector3D */
    public double getZ()
    {
        return z;
    }

    /** Return the square modulus of this Vector3D. This is equal
     * to (x*x) + (y*y) + (z*z). */
    public double getModulusSquared()
    {
	return (x * x) + (y * y) + (z * z);
    }

    /** Return the modulus of this Vector3D. This is equal to the
     * sqrt((x*x) + (y*y) + (z*z)). */
    public double getModulus()
    {
	return Math.sqrt((x * x) + (y * y) + (z * z));
    }

    /** Return a new Vector3D representing a unit vector in the direction of this 
      Vector3D. This is found by dividing all components by the magnitude. */
    public Vector3D getUnitVector()
    {
	double mod = getModulus();
	return new Vector3D(x / mod, y / mod, z / mod);
    }

    /** Return a new Vector3D in the opposite direction to this Vector3D. */
    public Vector3D negative()
    {
	return new Vector3D(-x, -y, -z);
    }

    /** Return a new Vector3D which is result of multiplying each
      component of this Vector3D by the scalar <i>d</i>. */
    public Vector3D multScalar(double d)
    {
	return new Vector3D(x * d, y * d, z * d);
    }

    /* The following Inplace methods were removed by Paul, 28/03/01 to make this
       class immutable. They have been moved to MutableVector3D, which is an
       extension of this class */

    /* Multiply each component of this Vector3D by the scalar <i>d</i>
      inplace. */
    /*
    // REMOVED
    public void multScalarInplace(double d)
    {
	x *= d;
	y *= d;
	z *= d;
    }
    */

    /** Return a new Vector3D representing the vector sum of this Vector3D
      and the Vector3D <i>v</i>. */
    public Vector3D add(Vector3D v)
    {
	return new Vector3D(x + v.x, y + v.y, z + v.z);
    }

    /* Increment this Vector3D by the Vector3D <i>v</i> inplace. */
    // REMOVED
    /*
    public void addInplace(Vector3D v)
    {
	x += v.x;
	y += v.y;
	z += v.z;
    }
    */

    /** Return a new Vector3D representing the vector difference between
      this Vector3D and the Vector3D <i>v</i>. */
    public Vector3D subtract(Vector3D v)
    {
	return new Vector3D(x - v.x, y - v.y, z - v.z);
    }

    /* Decrement this Vector3D by the Vector3D <i>v</i> inplace. */
    // REMOVED
    /*
    public void subtractInplace(Vector3D v)
    {
	x -= v.x;
	y -= v.y;
	z -= v.z;
    }
    */

    /** Return the scalar product of this Vector3D and the Vector3D <i>v</i>.
      This is a scalar equal to the product of the magnitudes of the two vectors times
      the cosine of the angle between them, and is also equal to the sum of the products of
      corresponding components. */
    public double dotProduct(Vector3D v)
    {
	return (x * v.x) + (y * v.y) + (z * v.z);
    }
    
    /** Return the cross product of this Vector3D and the Vector3D <i>v</i>. This is
    a vector normal to the plan containing the two vectors, with a magnitude equal
    to the product of the magnitudes of the two vectors times the sine of the angle
    between them, and is also equal to the determinent of the cartesian unit vectors
    and the components of the two vectors in row form. */
    public Vector3D crossProduct(Vector3D v)
    {
	return new Vector3D(y * v.z - z * v.y,
			  z * v.x - x * v.z,
			  x * v.y - y * v.x);
    }

    /** Return a string representation of this Vector3D formatted as (x y z) */
    public String toString()
    {
	return "(" + x + " " + y + " " + z + ")";
    }
}
