/**
 * Copyright (C) 2001  Paul Evans (see the file README for details on how
 * to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

/** Class to provide more useful string operations
 *
 * Written by Paul Evans
 * Last update: 18 May 2001
 */
public class StringOperations
{

    public static boolean isWhitespace(char c)
    {
	return ((c == ' ') || (c == '\n') || (c == '\t'));
    }

    public static String neatenString(String s)
	// removes leading and trailing whitespace
    {
	int len = s.length();
	int startPos = 0;
	while((startPos < len) && (isWhitespace(s.charAt(startPos))))
	    startPos++;

	if (startPos == len) {
	    // fell off end of string, so return empty string
	    return "";
	}

	int endPos = len - 1;

	// now we know there is at least 1 non-whitespace character in the string, so this won't
	// fall off the beginning
	while(isWhitespace(s.charAt(endPos)))
	    endPos--;

	return s.substring(startPos, endPos + 1);
    }

    private static final byte JUMPOVER_EVERYTHING = 0;
    // has no stopping condition - useful when testing a string for format errors

    private static final byte JUMPOVER_WHITESPACE = 1; 
    // moves the "cursor" over whitespace characters, stopping at the first non-whitespace

    private static final byte JUMPOVER_TOKEN = 2;
    // opposite of above

    private static final byte JUMPOVER_CHAR = 3;
    // keeps moving until required character is found

    private static int jumpOver(int startAt, String data, byte splitCondition, char splitChar) 
	throws StringOperationException
    {
	int braceLevel = 0;
	boolean inQuotes = false;
	boolean escaped = false;
	boolean inComment = false;

	int pos;
	char currentChar;

	if (startAt >= data.length())
	    return -1;

	// System.out.print("jumpOver: startAt=" + startAt + " ");

	// switch (splitCondition) {
	// case JUMPOVER_WHITESPACE:
	//     System.out.println("whitespace");
	//     break;
	// case JUMPOVER_TOKEN:
	//     System.out.println("token");
	//     break;
	// case JUMPOVER_CHAR:
	//     System.out.println("character " + splitChar);
	//     break;
	// }

	forloop: for (pos = startAt; pos < data.length(); pos++) {
	    currentChar = data.charAt(pos);
	    // System.out.println("jumpOver: pos=" + pos + " currentChar=" + currentChar);


	    // ignore comments first
	    switch (currentChar) {
	    case '#':
		if (!inQuotes && !escaped && !inComment)
		    inComment = true;
		break; // switch
	    case '\n':
		if (inQuotes && !escaped)
		    throw new StringOperationException("newline in string - to embed a newline character, please escape it", pos);
		inComment = false;
		break; // switch
	    } // end switch

	    // deal with splitCondition
	    if (!inQuotes && !escaped && !inComment && (braceLevel == 0)) {
		switch (splitCondition) {
		case JUMPOVER_EVERYTHING:
		    break; // switch
		case JUMPOVER_WHITESPACE:
		    if (!isWhitespace(currentChar)) { 
			// System.out.println("jumpOver: breaking because hit non-whitespace");
			break forloop;
		    }
		    break;
		case JUMPOVER_TOKEN:
		    if (isWhitespace(currentChar)) {
			// System.out.println("jumpOver: breaking because hit whitespace");
			break forloop;
		    }
		    break;
		case JUMPOVER_CHAR:
		    if (currentChar == splitChar) {
			// System.out.println("jumpOver: breaking because found splitChar");
			break forloop;
		    }
		    break;
		} // end switch
	    } // end if

	    // deal with other special characters
	    switch (currentChar) {
	    case '{': 
		if (!inQuotes && !inComment) braceLevel++; 
		break; // switch
	    case '}':
		if (!inQuotes && !inComment) braceLevel--;
		if (braceLevel < 0) throw new StringOperationException("more close braces than open braces", pos);
		break; // switch
	    case '\"':
		if (inComment) break;
		if (escaped) 
		    escaped = false;
		else 
		    inQuotes = !inQuotes;
		break; // switch
	    case '\\':
		if (!inComment)
		    escaped = !escaped;
		break; // switch
	    } // end switch

	} // end for

	// System.out.println("jumpOver: dropped out of for loop");

	// if fell off end of string don't send error, just point to end of string
	if (pos >= data.length()) {
	    // System.out.println("jumpOver: Fell off end of string - woooaaahhh <splash>");
	    if (inQuotes) throw new StringOperationException("string quote does not close", pos);
	    if (escaped) throw new StringOperationException("string ends on escape character", pos);
	    if (braceLevel != 0) throw new StringOperationException("string ends with braces still open", pos);
	    return data.length();
	}
	else 
	    return pos;
    }

    public static int jumpOverToken(int startAt, String s)
	throws StringOperationException
    {
	return jumpOver(startAt, s, JUMPOVER_TOKEN, '\0');
    }

    public static int jumpOverWhitespace(int startAt, String s)
	throws StringOperationException
    {
	return jumpOver(startAt, s, JUMPOVER_WHITESPACE, '\0');
    }

    public static int jumpOverToChar(int startAt, String s, char c)
	throws StringOperationException
    {
	return jumpOver(startAt, s, JUMPOVER_CHAR, c);
    }

    public static void verifyString(String s)
	throws StringOperationException
    {
	jumpOver(0, s, JUMPOVER_EVERYTHING, '\0');
    }

    public static String[] splitAtFirst(String s, char where)
	throws StringOperationException
	/* returns 2 element string array
	   [0] = up to but not including 'where'
	   [1] = including 'where' up to end of string
	   if 'where' not found then [0] = s, [1] = empty
	*/
    {
	int splitAt = jumpOver(0, s, JUMPOVER_CHAR, where);

	String[] ret = new String[2];
	if (splitAt == -1) {
	    ret[0] = s;
	    ret[1] = "";
	}
	else {
	    ret[0] = neatenString(s.substring(0, splitAt));
	    ret[1] = neatenString(s.substring(splitAt));
	}
	return ret;
    }

    public static String[] splitAtFirstWhitespace(String s)
	throws StringOperationException
	// similar to splitAtFirst, but breaks at any whitespace character
    {
	s = neatenString(s);
	int splitAt = jumpOver(0, s, JUMPOVER_TOKEN, '\0');

	String[] ret = new String[2];
	if (splitAt == -1) {
	    ret[0] = s;
	    ret[1] = "";
	}
	else {
	    ret[0] = neatenString(s.substring(0, splitAt));
	    ret[1] = neatenString(s.substring(splitAt));
	}
	return ret;
    }

    public static String convertEscapes(String s)
    {
	String ret = "";

	char thisChar;
	boolean escaped = false;

	for(int p = 0; p < s.length(); p++) {
	    thisChar = s.charAt(p);
	    if (escaped) {
		switch (thisChar) {
		case '\\': {
		    ret += "\\"; 
		    break;
		} // end case
		case '\"': { 
		    ret += "\"";
		    break;
		} // end case
		case 'n': {
		    ret += "\n";
		    break;
		} // end case
		case '\n': {
		    break; // ignore escaped newline chars
		} // end case
		} // end switch
		escaped = false;
	    }
	    else {
		switch (thisChar) {
		case '\\': {
		    escaped = true;
		    break;
		} // end case
		case '\"':
		    break; // ignore
		default:
		    ret += thisChar;
		} // end switch
	    }
	} // end for

	return ret;
    }

    public static String[] reportPosition(String s, int offset)
	/* returns an array of 3 strings;
	 * [0] = token at the offset, or empty string if that is a whitespace
	 * [1] = line number, as a decimal string
	 * [2] = character position within that line, as a decimal string
	 * if offset is not within string, returns (empty string, 0, 0)
	 */
    {

	String[] ret = {"","0","0"};
	
	if ((offset < 0) || (offset > s.length())) {
	    return ret;
	}

	int lineCount = 1, charCount = 0;
	String token = "";
	char thisChar = '\0';

	int charPos;

	// TODO: improve to take account of quoted strings, etc...

	for(charPos = 0; charPos <= offset; charPos++) {
	    thisChar = s.charAt(charPos);
	    if (thisChar == '\n') {
		lineCount++;
		charCount = 0;
	    }
	    else {
		charCount++;
	    }
	    if (isWhitespace(thisChar)) 
		token = "";
	    else 
		token += thisChar;
	    // end if
	} // end for

	// if we're currently on whitespace, then just move up until we're not
	if (isWhitespace(thisChar)) {
	    while(charPos < s.length()) {
		thisChar = s.charAt(charPos);
		if (isWhitespace(thisChar)) break;
		charPos++;
	    }
	}
	
	// now get rest of this token
	while(charPos < s.length()) {
	    thisChar = s.charAt(charPos);
	    if (isWhitespace(thisChar)) break;
	    token += thisChar;
	    charPos++;
	}
	       
	// return results
	ret[0] = token;
	ret[1] = "" + lineCount;
	ret[2] = "" + charCount;

	return ret;
    }
}
