/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


package noope.laws;

import noope.core.*;
import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.BRLoadingParseException;
import noope.input.BREntryNotFoundException;
import noope.entities.Entity;

/** Used to implement Newton's inverse square law of gravity
    @author The NOOPE team
    @version 26 March 2001, changed by Miklos Reiter to be subclass of ParticlePairLaw
 */
public class Gravity extends ParticlePairLaw
{
    /** This is the default universal gravitational constant having the value 6.6732e-11. */
    public static final double defaultG = 6.6732e-11;
    private double G; // universal gravitational constant

    /** Construct a Gravity Law from the supplied BlockReader, with
     the supplied lawnumber. After calling the inherited constructor, the 
     Gravity constructor tries to load G as the first Entry of BlockReader.
     If it fails, it assumes we're working with the defaultG constant, which
     has a value of 6.6732e-11. 
     @param br BlockReader used to initialise object
     @param lawnumber Number of this law
     */     
    public Gravity(BlockReader br, int lawnumber, Physics phys) throws BRLoadingException
    {
        super(br, lawnumber, phys);
        
        // get G from br:
        String[] brEntry; // store an entry from the BlockReader here temporarily

        // just peek first, for what if we are not supposed to be reading this...
        if (br.hasMoreEntries() && ((brEntry = br.peekNextEntry())[0] == "G"))
            {
                try
                    {
                        G = Double.valueOf(brEntry[1]).doubleValue();
                        br.getNextEntry(); // if we need the entry, advance pointer in br
                    }
                catch (NumberFormatException e)
                    {
                        throw new BRLoadingParseException("error while parsing for double!",
			           br.getEntryLocation());
                    }
            }
        else // take default value
            {
                G = defaultG;
            }
    }


    /** This evaluates the gravitational force for the given pair of entitys and calls
	addForce appropriately for each of them. */
    public void actOnPair(Entity ent1, Entity ent2)
    {
	// position vector of ent2 relative to ent1, direction of force on ent1
	Vector3D v1to2 = ent2.getPosition().subtract(ent1.getPosition());

	// now calculate force: F = G * m1 * m2 * x / (mod(x)^3)
	Vector3D F1 = v1to2.multScalar(G * 
					   (((GravityPropertyRecord)ent1.getPropertyRecord(getNumber())).
					    getGravitationalMass()) *
					   (((GravityPropertyRecord)ent2.getPropertyRecord(getNumber())).
					    getGravitationalMass()) /
					   (v1to2.getModulus() * v1to2.getModulusSquared()));
	ent1.addForce(F1);
	ent2.addForce(F1.negative());
    }

    /** Returns a new GravityPropertyRecord, calling its constructor with (br, ent). 
        The GravityPropertyRecord constructor then tries to load "gravitationalMass" 
	as the first entry of br; if it fails, it sets the gravitationalMass to the
        inertialMass of ent; if that fails, it raises an exception. */
    public LawPropertyRecord constructNewPropertyRecord(BlockReader br, Entity ent) throws BRLoadingException
    {
        LawPropertyRecord result = new GravityPropertyRecord(br, ent);
        if (br != null) br.assertEmpty(); // assert that either null or empty
        return result;
    }

    /** Return a string representation of this Gravity object, for debugging purposes. */
    public String toString()
    {
	return super.toString() + "\nG: " + G;
    }

}
