/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.laws;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.entities.Entity;

import noope.core.Vector3D;
import noope.core.Physics;

/** This law models collisions of spherical entitys.
    The entitys are assumed to be perfectly spherical with no spin.
    If collision is detected, it is carried out instantaneously, using conservation of momentum
    and Newton's coefficient of restitution. We are calling Entity.setVelocity(Vector3D) to do this.

    @author Miklos Reiter
    @version 27 March 2001
 */
public class SphereCollision extends ParticlePairLaw
{
    /** Initializes a new SphereCollision law object from the given BlockReader with the given
	lawnumber. */
    public SphereCollision(BlockReader br, int lawnumber, Physics phys)
    						throws BRLoadingException
    {
	super(br, lawnumber, phys);
    }

    /** Acts on the pair of entities ent1 and ent2. Uses addForce and step to change the Entity's
     velocity. Maybe, we can implement a Entity.setVelocity method in future.*/
    public void actOnPair(Entity ent1, Entity ent2)
    {
	Vector3D x1 = ent1.getPosition(), x2 = ent2.getPosition();
	Vector3D x1to2 = x2.subtract(x1); // x2 - x1
	double d2 = x1to2.getModulusSquared();
	
	double 
	    r1 = ((SphereCollisionPropertyRecord)ent1.getPropertyRecord(getNumber())).getRadius(),
	    r2 = ((SphereCollisionPropertyRecord)ent2.getPropertyRecord(getNumber())).getRadius(),
	    mindist = r1 + r2;
    
        Vector3D u1 = ent1.getVelocity(), u2 = ent2.getVelocity();

        // if the spheres are near enough, and they are approaching, then ...
	// approaching means:
	// dt = (mindist / (mod(u1) + mod(u2))) * 1e-6
	// mod (x1 + u1*dt - x2 - u2*dt) - mod(x1 - x2) < 0

	double dt = (mindist * 0.000001 / (u1.getModulus() + u2.getModulus()));

        if (d2 <= (mindist*mindist) && 
	    (x1to2.add(u2.multScalar(dt).subtract(u1.multScalar(dt))).getModulusSquared() 
	     < d2)
	    )
	    {
		double e = 1; // coefficient of restitution
		double m1 = ent1.getInertialMass(), m2 = ent2.getInertialMass();
		
		Vector3D L = x1to2.getUnitVector(); // unit vector along the line of centres
		
		// old components of velocities along the line of centres
		double u1L = u1.dotProduct(L), u2L = u2.dotProduct(L);
		
		// new components of velocities along the line of centres
		double v1L = (m1*u1L + m2*u2L + e*m2*(u2L - u1L))/(m1 + m2),
		    v2L = v1L - e*(u2L - u1L);

		
		// new velocities:
		Vector3D v1_new = u1.add(L.multScalar(v1L - u1L)),
		    v2_new = u2.add(L.multScalar(v2L - u2L));
		
		// set new velocites:
		ent1.setVelocity(v1_new);
		ent2.setVelocity(v2_new);

	    }
    }

    /** Returns a new SphereCollisionPropertyRecord. A SphereCollisionPropertyRecord stores the
	radius of the spherical Entity. */
    public LawPropertyRecord constructNewPropertyRecord(BlockReader br, Entity e) throws
	BRLoadingException
    {
	LawPropertyRecord result = new SphereCollisionPropertyRecord(br, e);
	br.assertEmpty(); // bail out if not all the information has been used for constructing
	// the LawPropertyRecord
	return result;
    }
}
