/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.awt.image.BufferedImage;
import java.awt.Graphics2D;

import java.io.OutputStream;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import net.jmge.gif.Gif89Encoder;

/**
 * This is an AnimationListener, that creates an animated gif of the frames it is given or static
 * gif frames of an animation.
 *
 * @author Miklos Reiter
 * @version 05-May-2001
 */
public class AnimatedGifCreator implements AnimationListener
{
    private boolean working;

    private BufferedImage im;
    private String fileName;
    private String comments;
    private int uniformDelay;
    private int loopCount;

    // the Graphics2D object returned by drawingGraphics().
    private Graphics2D g;

    private Gif89Encoder gifEnc;

    /** True if we are outputting a collection of static gif frames, rather than an animated gif.
	This is much better for memory consumption. */
    private boolean staticFrames;
    /** First and last part of filename in this case (frame number 0-n inserted between the two. */
    private String fileNameStart, fileNameEnd;
    /** The index of the next frame. */
    private int frameIndex = 0;

    /**
     * Constructs a new AnimatedGifCreator. After you have constructed it, you can draw into
     * the Graphics2D returned by drawingGraphics() to draw into the image held in this object.
     * After you've completed a frame, call updateComplete(). If you are using this object in
     * multiple threads, synchronize access to it, while drawing into the Graphics2D returned by
     * drawingGraphics. After using this object, you should call its dispose() method to write the
     * file.
     *
     * @arg aIm The image subsequently drawn into.
     * @arg aFileName The name of the file to store the animated gif to. If it contains a '*',
     * then a number of static gif images is output with the number inserted instead of the '*',
     * otherwise, an animated gif is produced. The former is much preferrable for reasons of
     * memory consumption (all the frames are kept in memory in the latter case).
     * @arg aComments The comments that should be put into the gif file.
     * @arg aUniformDelay The uniform delay (in centiseconds) put into the gif.
     * @arg aLoopCount How many times the animation should loop, default = 1, 0 = indefinitely.
     */
    public AnimatedGifCreator(BufferedImage aIm, String aFileName, String aComments, 
			      int aUniformDelay, int aLoopCount)
    {
	working = true;

	im = aIm;
	g = im.createGraphics();

	fileName = aFileName;
	int starIndex = (fileName.indexOf('*'));
	staticFrames = (starIndex != -1);
	if (staticFrames)
	    {
		fileNameStart = fileName.substring(0, starIndex);
		fileNameEnd = fileName.substring(starIndex + 1);
	    }
	
	comments = aComments;
	uniformDelay = aUniformDelay;
	loopCount = aLoopCount;
	
	gifEnc = new Gif89Encoder();
    }

    /** Returns a Graphics2D object that can be used to draw into the BufferedImage.
     */
    public Graphics2D drawingGraphics()
    {
	return g;
    }

    /** Should be called when frame complete, adds it to Gif89Encoder. This method is
	synchronized to prevent other threads from changing the BufferedImage in the mean time.
     */
    synchronized public void updateComplete()
    {
	// add our BufferedImage as a new frame
	try
	    {
		gifEnc.addFrame(im);
	    }
	catch (IOException e)
	    {
		System.err.println("Could not add frame to animation. " + e);
		e.printStackTrace();
	    }
	if (staticFrames)
	    {
		// format frame number as 4-digit number
		String frameNumber = Integer.toString(frameIndex);
		while (frameNumber.length() < 4) frameNumber = "0" + frameNumber;
		writeToGif(fileNameStart + frameNumber + fileNameEnd);
		gifEnc = new Gif89Encoder(); // release the current Gif89Encoder, together with its
		// frame
		frameIndex++;
	    }
    }

    /** Whether this AnimationListener is ready to be sent image information (or whether it is
	useful to do so. For example, a window, after being made invisible does not need any
	more graphics information.) Returns false after dispose() has been called.*/
    public boolean isWorking()
    {
	return working;
    }

    private void writeToGif(String file)
    {
	// set encoder attributes
	gifEnc.setComments(comments);
	gifEnc.setUniformDelay(uniformDelay);
	gifEnc.setLoopCount(loopCount);
	
	OutputStream out = null;

	// now encode...
	try
	    {
		out = new BufferedOutputStream(new FileOutputStream(file), 2048);
		
		try
		    {
			gifEnc.encode(out);
		    }
		catch (IOException e) // encode
		    {
			System.err.println("Could not encode gif. " + e);
			e.printStackTrace();
		    }
		out.close();
	    }
	catch (java.io.FileNotFoundException e)
	    {
		System.err.println("File " + fileName + " could not be opened or created. " + e);
		e.printStackTrace();
	    }
	catch (java.lang.SecurityException e)
	    {
                System.err.println("SecurityException occurred, seem not to have access to files."
				   + e);
                e.printStackTrace();
	    }
	catch (IOException e) // out.close()
	    {
		System.err.println("Could not close file. " + e);
		e.printStackTrace();
	    }
    }

    /** Encodes into a file. */
    public synchronized void dispose()
    {
	working = false;

	if (!staticFrames) writeToGif(fileName);
    }
}
