/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.awt.Frame;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

/**
 *
 * This class implements a frame object that is buffered: whenever it needs redrawing, it occurs
 * into a BufferedImage object first, before it is finally drawn to the screen. This reduces
 * flickering if frequent redraws occur, e.g. in OutputProjection. Although using Swing would
 * do the job, most browsers are still not Swing-capable.
 *
 * @author Miklos Reiter
 * @version 14-Aug-2001
 * changed by Miklos Reiter on August 14, 2001 to use
 * insets correctly to determine the size of the frame;
 * in fact, the old code should have created too big a buffer,
 * but using some Linux window managers (twm, fvwm2),
 * the insets values
 * were negative and the raw frame size too small, however
 * the net frame size was right!
 */
public class BufferedFrame extends Frame
{
    /** The buffer used for drawing. */
    protected Image buffer;

    /** Constructs a new BufferedFrame.
     *
     * @arg title The String to use as the title of the window.
     * @arg width The width of the window.
     * @arg height The height of the window.
     */
    public BufferedFrame(String title, int width, int height)
    {
	super(title);
	buffer = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
    }

    /** Redraws the frame. Overridden to draw into a buffer first. If the height or width of the
	frame has changed, a new buffer is created with the new width and height.
     */
    public void update(Graphics g)
    {
	try
	    {
		// changed by Miklos Reiter on August 14, 2001 to use
		// insets correctly to determine the size of the frame;
		// in fact, the old code should have created too big a buffer,
		// but using some Linux window managers (twm, fvwm2),
		// the insets values
		// were negative and the raw frame size too small, however
		// the net frame size was right!
	    	int height = getHeight() - getInsets().top - getInsets().bottom;
		int width = getWidth() - getInsets().left - getInsets().right;
		if ((buffer.getHeight(null) != height) || (buffer.getWidth(null) != width))
		    {
			buffer = createImage(width, height);
		    }
		// update into buffer
		super.update(buffer.getGraphics());
		// now draw the buffer
		g.drawImage(buffer, 0, 0, null, null);
	    }
	catch (java.lang.NullPointerException e) // the window is already being closed while we are
	    {// drawing, so don't worry about it
	    }
    }

    /** Returns true. */
    public boolean isDoubleBuffered()
    {
	return true;
    }
    
    /** Hides the frame and disposes of system resources held by it, making it unusable. */
    public void dispose()
    {
	// hide
	setVisible(false);
	// now destroy data
	if (buffer != null)
	    {
		buffer.flush();
		buffer = null;
	    }
	super.dispose();
    }
}
