/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.awt.Component;
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Dimension;

import java.lang.Thread;

/** This class displays a BufferedImage on the screen. To prevent graphics mess-ups, access objects
    of this class in a synchronized way.
    
    @author Miklos Reiter
    @version 26-Apr-2001: added get{Preferred,Minimum,Maximum}Size() methods [Miklos];
    22-Apr-2001 changed by Miklos Reiter to use synchronized access and not two
    BufferdImage objects: this is faster and more reliable; dispose() added by Miklos Reiter
    on 19-Apr-2001
*/
public class DrawingWindow extends Component implements AnimationListener
{
    private BufferedImage image; // image to display
    private Graphics2D graphics; // Graphics2D object used to draw into image

    /** Stores the minimum/preferred/maximum dimension of this component. */
    private Dimension dim;

    private boolean disposed;

    /** Constructs a DrawingWindow object that displays the given BufferedImage. To use this
	you have to then insert it into a frame and display it on the screen.
    */
    public DrawingWindow(BufferedImage im)
    {
	super(); // call inherited constructor

	disposed = false;

	image = im;

	// set dimension of image
	dim = new Dimension(im.getWidth(), im.getHeight());

	graphics = image.createGraphics();
    }
    
    public Dimension getMinimumSize()
    {
	return dim;
    }

    public Dimension getPreferredSize()
    {
	return dim;
    }

    public Dimension getMaximumSize()
    {
	return dim;
    }

    /** Draws the BufferedImage the DrawingWindow has been constructed with onto the screen. */
    synchronized public void paint(Graphics g) // just draw image
    {
	g.drawImage(image, 0, 0, Color.white, null);
    }
    
    /** Returns a Graphics2D object that can be used to draw into the BufferedImage.
     */
    public Graphics2D drawingGraphics()
    {
	return graphics;
    }


    /** This method calls repaint(). */
    public void updateComplete()
    {
	repaint();
    }

    /** Disposes of system resources used. Added by Miklos Reiter on 19-Apr-2001. */
    public void dispose()
    {
	disposed = true;
	try
	    {
		graphics.dispose();
		graphics = null;
		image.flush();
		image = null;
	    }
	catch (java.lang.NullPointerException e)
	    {}
    }
    
    /** False after dispose() has been called. */
    public boolean isWorking()
    {
	return (!disposed);
    }
}
